'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function (_ref) {
  var t = _ref.types;

  var generateElement = function generateElement(path, state) {
    var FILE = state.file,
        OPTIONS = _extends({}, { type: 'type', 'extends': 'extends', attributes: 'attributes', children: 'children' }, state.opts);

    var NODE = path.node;

    if (!/JSXElement/.test(NODE.type)) return NODE.expression ? NODE.expression : t.StringLiteral(NODE.value);

    var OPENING_ELEMENT = NODE.openingElement,
        CHILDREN = path.get('children'),
        ELEMENT_ATTRIBUTES = OPENING_ELEMENT.attributes,
        EXTENDS = isExtension(OPENING_ELEMENT, path);

    var type = t.StringLiteral(OPENING_ELEMENT.name.name),
        attributes = ELEMENT_ATTRIBUTES.length ? buildAttributeObject(ELEMENT_ATTRIBUTES, FILE) : t.NullLiteral(),
        children = CHILDREN.length ? t.ArrayExpression(CHILDREN.map(function (child) {
      return generateElement(child, state);
    })) : t.NullLiteral();

    return t.ObjectExpression([t.ObjectProperty(t.StringLiteral(OPTIONS.extends), EXTENDS ? t.Identifier(OPENING_ELEMENT.name.name) : t.NullLiteral()), t.ObjectProperty(t.StringLiteral(OPTIONS.type), EXTENDS ? t.NullLiteral() : type), t.ObjectProperty(t.StringLiteral(OPTIONS.attributes), attributes), t.ObjectProperty(t.StringLiteral(OPTIONS.children), children)]);
  };

  var generateAttrObject = function generateAttrObject(nodes) {
    var arr = nodes.map(function (node) {
      var name = t.StringLiteral(node.name.name);
      var value = !node.value ? t.BooleanLiteral(true) : /JSXExpressionContainer/i.test(node.value.type) ? node.value.expression : node.value;

      return t.ObjectProperty(name, value);
    });

    return [t.ObjectExpression(arr)];
  };

  var buildAttributeObject = function buildAttributeObject(attrs, file) {
    var _expressions = [],
        _spreads = [];

    while (attrs.length) {
      var attr = attrs.shift();

      /^JSXSpreadAttribute$/i.test(attr.type) ? _spreads.push(attr.argument) : _expressions.push(attr);
    }

    var attrObject = _expressions.length ? generateAttrObject(_expressions) : null;

    if (_spreads.length) {
      var extension = attrObject ? _spreads.concat(attrObject) : _spreads;

      if (extension.length > 1) extension.unshift(t.ObjectExpression([]));

      attrObject = t.callExpression(file.addHelper('extends'), extension);
    } else {
      attrObject = attrObject[0];
    }

    return attrObject;
  };

  var isExtension = function isExtension(openingElement, path) {
    return path.scope.hasBinding(openingElement.name.name);
  };

  return {
    inherits: require('babel-plugin-syntax-jsx'),
    visitor: {
      JSXElement: function JSXElement(path, state) {
        path.replaceWith(generateElement(path, state));
      }
    }
  };
};